"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.freezeTokenPaymentGuardManifest = void 0;
const mpl_essentials_1 = require("@metaplex-foundation/mpl-essentials");
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const errors_1 = require("../errors");
const generated_1 = require("../generated");
exports.freezeTokenPaymentGuardManifest = {
    name: 'freezeTokenPayment',
    serializer: generated_1.getFreezeTokenPaymentSerializer,
    mintParser: (context, mintContext, args) => {
        const freezeEscrow = (0, generated_1.findFreezeEscrowPda)(context, {
            destination: args.destinationAta,
            candyMachine: mintContext.candyMachine,
            candyGuard: mintContext.candyGuard,
        });
        const nftAta = (0, mpl_essentials_1.findAssociatedTokenPda)(context, {
            mint: mintContext.mint,
            owner: mintContext.minter.publicKey,
        });
        const tokenAddress = (0, mpl_essentials_1.findAssociatedTokenPda)(context, {
            mint: args.mint,
            owner: mintContext.minter.publicKey,
        });
        const freezeAta = (0, mpl_essentials_1.findAssociatedTokenPda)(context, {
            mint: args.mint,
            owner: freezeEscrow,
        });
        return {
            data: new Uint8Array(),
            remainingAccounts: [
                { publicKey: freezeEscrow, isWritable: true },
                { publicKey: nftAta, isWritable: false },
                { publicKey: tokenAddress, isWritable: true },
                { publicKey: freezeAta, isWritable: true },
            ],
        };
    },
    routeParser: (context, routeContext, args) => {
        const { path } = args;
        switch (path) {
            case 'initialize':
                return initializeRouteInstruction(context, routeContext, args);
            case 'thaw':
                return thawRouteInstruction(context, routeContext, args);
            case 'unlockFunds':
                return unlockFundsRouteInstruction(context, routeContext, args);
            default:
                throw new errors_1.UnrecognizePathForRouteInstructionError('freezeTokenPayment', path);
        }
    },
};
const initializeRouteInstruction = (context, routeContext, args) => {
    const freezeEscrow = (0, generated_1.findFreezeEscrowPda)(context, {
        destination: args.destinationAta,
        candyMachine: routeContext.candyMachine,
        candyGuard: routeContext.candyGuard,
    });
    const freezeAta = (0, mpl_essentials_1.findAssociatedTokenPda)(context, {
        mint: args.mint,
        owner: freezeEscrow,
    });
    const systemProgram = context.programs.getPublicKey('splSystem', '11111111111111111111111111111111');
    const tokenProgram = context.programs.getPublicKey('splToken', 'TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA');
    const associatedTokenProgram = context.programs.getPublicKey('splAssociatedToken', 'ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL');
    const s = context.serializer;
    const serializer = s.tuple([
        (0, generated_1.getFreezeInstructionSerializer)(context),
        s.u64(),
    ]);
    return {
        data: serializer.serialize([generated_1.FreezeInstruction.Initialize, args.period]),
        remainingAccounts: [
            { publicKey: freezeEscrow, isWritable: true },
            { signer: args.candyGuardAuthority, isWritable: false },
            { publicKey: systemProgram, isWritable: false },
            { publicKey: freezeAta, isWritable: true },
            { publicKey: args.mint, isWritable: false },
            { publicKey: tokenProgram, isWritable: false },
            { publicKey: associatedTokenProgram, isWritable: false },
            { publicKey: args.destinationAta, isWritable: true },
        ],
    };
};
const thawRouteInstruction = (context, routeContext, args) => {
    const freezeEscrow = (0, generated_1.findFreezeEscrowPda)(context, {
        destination: args.destinationAta,
        candyMachine: routeContext.candyMachine,
        candyGuard: routeContext.candyGuard,
    });
    const nftAta = (0, mpl_essentials_1.findAssociatedTokenPda)(context, {
        mint: args.nftMint,
        owner: args.nftOwner,
    });
    const nftEdition = (0, mpl_token_metadata_1.findMasterEditionPda)(context, { mint: args.nftMint });
    const tokenProgram = context.programs.getPublicKey('splToken', 'TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA');
    const tokenMetadataProgram = context.programs.getPublicKey('mplTokenMetadata', 'metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s');
    return {
        data: (0, generated_1.getFreezeInstructionSerializer)(context).serialize(generated_1.FreezeInstruction.Thaw),
        remainingAccounts: [
            { publicKey: freezeEscrow, isWritable: true },
            { publicKey: args.nftMint, isWritable: false },
            { publicKey: args.nftOwner, isWritable: false },
            { publicKey: nftAta, isWritable: true },
            { publicKey: nftEdition, isWritable: false },
            { publicKey: tokenProgram, isWritable: false },
            { publicKey: tokenMetadataProgram, isWritable: false },
        ],
    };
};
const unlockFundsRouteInstruction = (context, routeContext, args) => {
    const freezeEscrow = (0, generated_1.findFreezeEscrowPda)(context, {
        destination: args.destinationAta,
        candyMachine: routeContext.candyMachine,
        candyGuard: routeContext.candyGuard,
    });
    const freezeAta = (0, mpl_essentials_1.findAssociatedTokenPda)(context, {
        mint: args.mint,
        owner: freezeEscrow,
    });
    const tokenProgram = context.programs.getPublicKey('splToken', 'TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA');
    const systemProgram = context.programs.getPublicKey('splSystem', '11111111111111111111111111111111');
    return {
        data: (0, generated_1.getFreezeInstructionSerializer)(context).serialize(generated_1.FreezeInstruction.UnlockFunds),
        remainingAccounts: [
            { publicKey: freezeEscrow, isWritable: true },
            { signer: args.candyGuardAuthority, isWritable: false },
            { publicKey: freezeAta, isWritable: true },
            { publicKey: args.destinationAta, isWritable: true },
            { publicKey: tokenProgram, isWritable: false },
            { publicKey: systemProgram, isWritable: false },
        ],
    };
};
//# sourceMappingURL=freezeTokenPayment.js.map